#!/usr/bin/env python
# Copyright (C) Etienne Robillard 2007-2009 <robillard.etienne@gmail.com>
# All rights reserved.
"""
Elixir Controller API

Helper classes to implement a generic WSGI handler supporting the 
default Elixir ORM backend. 
"""

from __future__ import absolute_import
from . base     import BaseController

import elixir

__all__ = ('ElixirControllerMixin', 'ElixirController')


class ElixirControllerMixin(BaseController):
    """ 
    A ``BaseController`` subclass supporting contextual sessions
    on top of Elixir/SQLAlchemy.

    """

    engine = None
 
    def __init__(self, global_conf, **kwds):
        """Setup the default Elixir parameters here"""
        BaseController.__init__(self, global_conf, **kwds)

        # Configure the Engine instance 
        if getattr(self, 'engine'):
            print "%s: found engine: %r" % (self, self.engine)
            self.init_session(self.engine)

    def init_session(self, engine, shortnames=True):
        """ Prepare the session stuff with Elixir """
        
        from notmm.utils.sql.session import ScopedSession as scoped_session
        scoped_session.set_session(engine, autoflush=True, autocommit=False)
    
        ## replace the elixir session with our own
        ## http://cleverdevil.org/computing/68/
        elixir.session = scoped_session.get_session()
        elixir.options_defaults.update(dict(shortnames=bool(shortnames)))
    
        # setup_all 
        if not elixir.metadata.is_bound(): 
            elixir.metadata.bind = engine
    
        # create tables if they're missing...
        # meta.create_all()
        
        # copy elixir.session to self.session
        # setattr(self, '_session', elixir.session)
    
    def __call__(self, environ, start_response):

        try:
            return self.application(environ, start_response)
        finally:
            elixir.session.close()

class ElixirController(ElixirControllerMixin):
    pass
            
