#!/usr/bin/env python
# -*- coding: utf-8 -*-
# filename: generic.py
"""Generic views for rendering templates with Mako and WebOb"""

 
__all__ = ('render_to_response', )

from notmm.utils.response   import WSGIResponse
from mako.exceptions        import TemplateLookupException

import configuration

def render_to_response(request, template_name, **kwds):
    """
    Generic view for returning a Mako template inside a 
    simple ``WSGIResponse`` instance.

    """

    # TemplateLoaderMiddleware:
    # l = request.environ['wsgiapp.template_loader']
    # settings = request.settings
    
    # attempt to get the list of template directories using 
    # the TemplateLoaderMiddleware strategy, or else fallback
    # on the django.conf mecanism.
    try:
        directories = request.environ['wsgiapp.template_dirs']
    except KeyError:
        if not hasattr(request, 'settings'):
            from django.conf import settings
            directories = settings.TEMPLATE_DIRS
        else:
            # This is not going to work unless SettingsMiddleware
            # is being implemented
            directories = request.settings.TEMPLATE_DIRS

    defaultLoader = configuration.defaultLoaderClass(directories)
    
    try:
        t = defaultLoader.get_template(template_name)
    except TemplateLookupException, e:
        status_code = 500
#       outstream = str(e)
        raise TemplateLookupException(e)
    else:
    	try:
            # Render the template to a string
            chunk = t.render(data=configuration.defaultRequestClass(request, kwds))
        except KeyError, e:
            raise KeyError(e)
        
    if not kwds.has_key('status_code'):
        status_code = 200
    else:
        status_code = kwds['status_code']

    # Handle the content type. Attempt to guess a 
    # good content-type if not already set.
    if not kwds.has_key('mimetype'):
        content_type = configuration.defaultContentType
    else:
        content_type = str(kwds['mimetype'])
        
    # Translates webob.Response params directly
    headerList = [('Content-Type', content_type)]
    
    return WSGIResponse(headerlist=headerList, status=status_code,
        app_iter=iter([chunk,])
        )

