#!/usr/bin/env python
"""Functional utilities for parsing configuration files

"""

# Copyright (C) 2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.

import os
from configobj import ConfigObj

__all__ = ('boolgetter', 
           'confgetter',
           'has_development_ini',
           'string_getter', 
           'string_getter_plus'
           'loadconf', 
           'getconf',
           'true',
           'false')

_false = "false"
_true = "true"

### lambda functions (shortcuts)
boolgetter = lambda x: bool(x)

# string_getters 
string_getter = lambda c,section,key: c[section].get(key, None)
string_getter_plus = lambda c,section=None: c[section] or c
false = lambda s: s.__eq__(_false)
true = lambda s: s.__eq__(_true)
confgetter = string_getter # kept back for backward compatibility

make_path = lambda relname: os.path.realpath (os.path.join (os.getcwd(), relname))
#is_readable = bool(lambda x, filename: os.access (os.path.join (x, filename), os.R_OK))
has_development_ini = lambda x: os.access (os.path.join (x, 'development.ini'), os.R_OK)

def loadconf(filename, section=None):
    """ 
    Returns a fully configured ConfigObj instance.

    """
    conf = ConfigObj(make_path(filename))
    if conf.has_key(section):
        return conf[section]
    else:
        return conf   

def getconf(relname):
    """ load a configuration file and return its content """
    # Default options are now located in development.ini.
    return loadconf(make_path(relname))

