#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2008 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
# 
"""Several HTTP-related extras on top of Paste"""

from datetime          import datetime
from paste.script      import wsgiserver
from paste.translogger import TransLogger

try:
    from django.utils.termcolors import make_style
except ImportError:
    pass

from notmm.utils.configuration import string_getter

__all__ = ('Banner', 'get_bind_addr', 
           'make_server', 'setup_http_server')

class Banner(object):
    """This is a simple startup banner.

    Usage:
        >>> banner = Banner(sys.stderr, ('127.0.0.1', '8000'))
        >>> banner.show(opts=('bold',), fg="blue")
    """

    default_format = "%Y-%m-%d at %H:%M:%S %p %Z"

    def __init__(self, stream, **kwargs):
        #self.stream = logging.StreamHandler(stream)
        self.stream = stream
        self.bind_addr = kwargs.get('bind_addr', None)
        self.fmt = kwargs.get('fmt', self.default_format)
        if not isinstance(self.bind_addr, tuple):
            raise ValueError("Error: invalid bind_addr type: %r" % type(self.bind_addr))

        assert len(self.bind_addr) == 2, 'bind_addr must contains exactly two elements'

    def show(self, opts=(), **kwargs):
        """Override this method to handle the formatting of the banner"""
        s = getattr(self, 'stream') 
        
        # get a color picker if make_style is available
        try:
            colorpicker = make_style(opts, **kwargs)
        except NameError:
            colorpicker = unicode
        
        print >> s, colorpicker("Initializing on %s" % datetime.today().strftime(self.fmt))
        print >> s, colorpicker("Starting HTTP server on host %s:%d" % self.bind_addr)
        
        return self

def get_bind_addr(app_conf, section):
    """
    Return a 2-element tuple containing the hostname and port
    to listen for remote connections.
    """

    host = string_getter(app_conf, section, 'host') or 'localhost'
    port = string_getter(app_conf, section, 'port') or 8000

    return (host, int(port))

def make_server(wsgi_app, bind_addr, **kwargs):
    """Create an instance of a threaded WSGI server"""
    
    # translogger
    logged_app = TransLogger(wsgi_app)
    
    server = wsgiserver.CherryPyWSGIServer(bind_addr,
        logged_app, **kwargs)


    return server

setup_http_server = make_server
