#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
#
# This file is part of the notmm project.
# More informations @ http://gthc.org/projects/notmm/

import webob
import webob.exc

__all__ = ('WSGIResponse', 
           'HTTPResponse', 
           'HTTPRedirectResponse')

class WSGIResponse(webob.Response):
    """ Wraps `webob.Response` in depth."""
    
    def __init__(self, *args, **kwds):
        """ Init the ``webob.Response`` object. """
       	webob.Response.__init__(self, *args, **kwds)

        # Use app_iter if this iterator is available
        if not hasattr(self, 'app_iter'):
            self._iter = getattr(self, 'body')
        else:
            self._iter = getattr(self, 'app_iter')

    def __iter__(self):
        """ Return a custom iterator type. """
        return iter(self._iter)
    
    def __len__(self):
        """ Return the Content-length value (type int).

        This value should match the value of ``self.content_length``.
        """
        return len(self.body)

    def next(self):
        """ required method for being a proper iterator type """
        chunk = self._iter.next()
        if isinstance(chunk, unicode):
            return chunk.encode(self.charset)
        return chunk
    
    def close(self):
        """ close the output stream """
        # XXX: validate if this method is forward-compatible with Py3k.
        if hasattr(self._iter, 'close'):
            self._iter.close()
        return None

HTTPResponse = WSGIResponse

class HTTPRedirectResponse(webob.exc.HTTPFound):
    """For routing HTTP requests to alternate locations"""

