#!/usr/bin/env python
# -*- coding: utf-8 -*-
# $Id$
"""
This module provides a simple mecanism for getting a per-thread 
contextual session object.

For basic usage, simply import the ``ScopedSession`` object and
call ``set_session``. This instance method  requires a ``engine``
object as its first argument. Once this step is done the scoped
session object should be bound to your engine and ready to use. 
"""
 
from threading      import local
from sqlalchemy.orm import scoped_session, sessionmaker

__all__ = ['ThreadLocalSession', 'ScopedSession']

class ThreadLocalSession(local):
    """  Contextual session wrapper """
 
    def __init__(self):
        self._session = None
 
    def get_session(self):
        return getattr(self, '_session')

    def set_session(self, engine, autoflush=True, autocommit=False):
        """ Initialize a regular ``Session`` object. """
        local_session = scoped_session(sessionmaker(autoflush=autoflush, \
            autocommit=autocommit))
        # configure the Session object
        local_session.configure(bind=engine)
        self._session = local_session
 
    session = property(get_session, set_session)

ScopedSession = ThreadLocalSession()

