#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2008,2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.

""" Helper functions useful to web application testing 

"""
import sys, os

from unittest import TextTestRunner
from notmm.utils.configuration import confgetter
from collection2 import TestCollection, get_test_modules

__all__ = ('SimpleTestRunner', )

class SimpleTestRunner(TextTestRunner):
    """
    A simplified ``unittest.TextTestRunner`` subclass with 
    RFC-822 style configuration.

    """
    
    def __init__(self, app_conf, app_label='collections'):
        """ Setup default options for the TestRunner object. 
        
        XXX: Put more documentation here.
        """
        self.app_conf = app_conf
        self.app_label = app_label

    def __call__(self, **kwargs):

        return self.run(**kwargs)

    #def set_app_label(self, keyname):
    #    """Set the default config section to look in for testing options"""
    #    setattr(self, 'app_label', keyname)
    #
    #    return self

    def run(self, **kwargs):
        """Run the test suite using ``TextTestRunner``"""

        app_conf = getattr(self, 'app_conf')
        app_label = getattr(self, 'app_label')

        collections = confgetter(app_conf, app_label, 'collections')
        exclude = confgetter(app_conf, app_label, 'exclude')
        verbosity = confgetter(app_conf, app_label, 'verbosity')
        extra_libs = confgetter(app_conf, app_label, 'extra_libs')
        #import pdb; pdb.set_trace()
        
        # add any extra paths to sys.path
        sys.path.extend([dirname for dirname in extra_libs.split() if \
            os.path.exists(dirname) ])

        loader = TestCollection() 
        grapes = get_test_modules(collections)
        suite = loader.loadTestsFromList(grapes)
        return TextTestRunner(verbosity=int(verbosity)).run(suite)

